"use strict";
// https://playwright.dev/docs/network
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const request_1 = require("./request");
const response_1 = require("./response");
const logger_1 = tslib_1.__importDefault(require("../logger"));
const utils_1 = require("../utils");
const url_1 = require("url");
/**
 * There is no valid reason to have more than one driver instances per page
 */
const pagesSet = new Set();
const redirectCodes = [301, 302, 303, 305, 307, 308];
class PlaywrightDriver {
    constructor({ page, noParseResponseUrls = [] }) {
        this._routeUrl = () => true;
        logger_1.default.debug(`instantiating new playwright driver`);
        if (!page) {
            throw new Error('"page" is not defined');
        }
        if (pagesSet.has(page)) {
            logger_1.default.error(`second driver instantiation on the same page`);
            logger_1.default.error(`(probably you did not stop the mocker before start it again)`);
            throw new Error(`second driver instantiation on the same page`);
        }
        this.seenRedirects = new Map();
        this.noParseResponseUrls = noParseResponseUrls;
        this.page = page;
        pagesSet.add(page);
        this.page.on('close', () => {
            pagesSet.delete(this.page);
        });
    }
    async setRequestInterception(_arg) {
        // do nothing for playwright
    }
    async onRequest(fn) {
        const handler = async (route) => {
            const timestampWithOrder = (0, utils_1.getTimeStampWithStrictOrder)();
            (0, utils_1.loggerTrace)(`${route.request().url()} ← page.on('request') fired`);
            const { request, abort, next, respond } = (0, request_1.extractPlaywrightRequest)(route, timestampWithOrder);
            await fn({ request, abort, next, respond });
        };
        await this.page.route(this._routeUrl, handler);
        return async () => {
            pagesSet.delete(this.page);
            try {
                await this.page.unroute(this._routeUrl, handler);
            }
            catch (e) {
                if (e.message.indexOf('has been closed') === -1) {
                    throw e;
                }
            }
        };
    }
    async onResponse(fn) {
        const handler = async (interceptedResponse) => {
            const timestampWithOrder = (0, utils_1.getTimeStampWithStrictOrder)();
            const url = interceptedResponse.request().url();
            if (redirectCodes.indexOf(interceptedResponse.status()) !== -1) {
                // see https://playwright.dev/docs/api/class-page#pagerouteurl-handler
                // Because of the way the page.route method works, the following redirection will not be handled by it.
                const newLocation = interceptedResponse.headers().location;
                const destUrl = new url_1.URL(newLocation, url).toString();
                this.seenRedirects.set(destUrl, (this.seenRedirects.get(destUrl) || 0) + 1);
                logger_1.default.debug(`Redirection to "${newLocation}" from "${url}" will not be handled.`);
            }
            if (this.seenRedirects.get(url)) {
                this.seenRedirects.set(url, this.seenRedirects.get(url) - 1);
                return;
            }
            const noParseResponse = this.noParseResponseUrls.some((urlPart) => url.indexOf(urlPart) !== -1);
            (0, utils_1.loggerTrace)(`${url} → page.on('response') fired`);
            await fn(await (0, response_1.extractPlaywrightResponse)(interceptedResponse, {
                ...timestampWithOrder,
                noParseResponse,
            }));
            (0, utils_1.loggerTrace)(`${url} → finish`);
        };
        // Intercepting all requests and responding with mocks
        await this.page.on('response', handler);
        return () => this.page.off('response', handler);
    }
}
exports.default = PlaywrightDriver;
