"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getFileName = void 0;
const tslib_1 = require("tslib");
const fs_1 = require("fs");
const path_1 = tslib_1.__importDefault(require("path"));
const make_dir_1 = tslib_1.__importDefault(require("make-dir"));
const debug_1 = tslib_1.__importDefault(require("debug"));
const logger_1 = tslib_1.__importDefault(require("./logger"));
const os = require('os');
const pathToUnixPath = os.platform() === 'win32' ? (str) => str.replace(/\\/g, '/') : (str) => str;
const loggerGet = (0, debug_1.default)('teremock:storage:get');
const loggerSet = (0, debug_1.default)('teremock:storage:set');
const loggerHas = (0, debug_1.default)('teremock:storage:has');
const getFileName = ({ wd, mockId }) => {
    return pathToUnixPath(path_1.default.resolve(wd, mockId.replace('--', path_1.default.sep) + '.json'));
};
exports.getFileName = getFileName;
class FileStorage {
    constructor(params) {
        this.wd = params?.wd ?? path_1.default.resolve(process.cwd(), '__teremocks__');
    }
    async set(mockId, json) {
        loggerSet(`entering storage.set with mockId "${mockId}"`);
        const absFileName = (0, exports.getFileName)({ mockId, wd: this.wd });
        const targetDir = path_1.default.dirname(absFileName);
        const content = JSON.stringify(json, null, '  ');
        await (0, make_dir_1.default)(targetDir);
        loggerSet(`successfully checked/created targetDir ${targetDir}`);
        if (await this.has(mockId)) {
            loggerSet(`file already exists, overwriting ${mockId}`);
        }
        logger_1.default.set(`writing mock for mockId "${mockId}", url "${json?.request?.url}"`);
        logger_1.default.set(`to file "${absFileName}"`);
        try {
            await fs_1.promises.writeFile(absFileName, content);
            logger_1.default.success(`successfully wrote new file ${absFileName}`);
        }
        catch (e) {
            logger_1.default.error(`failed to write new file ${absFileName}`);
            throw e;
        }
    }
    // only for tests
    _getFn(mockId) {
        return (0, exports.getFileName)({ mockId, wd: this.wd });
    }
    async get(mockId) {
        const absFileName = this._getFn(mockId);
        loggerGet(`about to read file ${absFileName}`);
        try {
            const data = JSON.parse(await fs_1.promises.readFile(absFileName, 'utf8'));
            loggerGet(`successfully read the file ${absFileName}`);
            return data;
        }
        catch (err) {
            if (err.code === 'ENOENT') {
                loggerGet(`fail to read the file (ENOENT) ${absFileName}`, err.message);
            }
            else {
                logger_1.default.error(`fail to read the file ${absFileName} for some other reason`, err);
            }
            throw err;
        }
    }
    async has(mockId) {
        const absFileName = this._getFn(mockId);
        loggerHas(`about to check file ${absFileName}`);
        let fileExists = false;
        try {
            await fs_1.promises.access(absFileName);
            fileExists = true;
            loggerHas(`file "${absFileName}" exists`);
        }
        catch (e) {
            loggerHas(`file "${absFileName}" does not exists`);
        }
        return fileExists;
    }
    setWd(nextWd) {
        this.wd = typeof nextWd === 'string' ? nextWd : path_1.default.join(process.cwd(), ...nextWd);
        return this.wd;
    }
}
exports.default = FileStorage;
